

/*======================================================================

    PCI30HX, CB31 scsi configuration

    ascconf.c
        0.70 2001/10/26            Add PCI Burst bit
	0.61 2001/03/05 16:22:50
    	0.60 2000/08/03 11:49:50

    The contents of this file are subject to the Mozilla Public
    License Version 1.1 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.

    The initial developer of the original code is RATOC Systems, Inc.
    <www.rexpccard.co.jp>.  Portions created by RATOC Systems, Inc.
    are Copyright (C) 2000 RATOC Systems,Inc.  All Rights Reserved.

    Alternatively, the contents of this file may be used under the
    terms of the GNU Public License version 2 (the "GPL"), in which
    case the provisions of the GPL are applicable instead of the
    above.  If you wish to allow the use of your version of this file
    only under the terms of the GPL and not to allow others to use
    your version of this file under the MPL, indicate your decision
    by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL.  If you do not delete
    the provisions above, a recipient may use your version of this
    file under either the MPL or the GPL.

======================================================================*/

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>

#ifdef __GLIBC__
#include <sys/io.h>
#else
#include <asm/io.h>
#endif

#define _GNU_SOURCE
#include <getopt.h>

// #include <linux/delay.h>	2001.03.05
#include <linux/pci.h>

#include "eepasc.h"

typedef struct ascconf_s {
    int   eep00;        /* +00 data save */
    int   pburst_v;     /* PCI Burst */
    uchar disc_v[8];    /* Disconnect */
    uchar sync_v[8];    /* Transfer rate */
    uchar stdr_v[8];    /* Sync enable/disable */
} ASCCONF_T;

static char *this_program = "PCI30HX, CB31 scsi configuration utility (%s): ";
static char *this_version = "Beta Version 0.70 2001/10/26";

static int dbgf = 0;	/* debug flag  0:normal  1,2,3...:debug */
static int verbose = 0;
static int hex_show = 0;

static int disc_chg = 0;	/* 0: no change  1:change */
static int sync_chg = 0;	/* 0: no change  1:change */
static int pburst_chg = 0;      /* 0: no change  1:change */

static uchar disc_val[8]
	 = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
		/* 0:disconnect disable  1:enable  0xFF:no set */
static uchar sync_val[8]
	 = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
		/* 0:Async  5:5MB  10:10MB  20:20MB  0xFF:no set */
static uchar pburst_val;         /* 0: off  1: on */

static int io_base;
static int irq_no;

static ASCCONF_T  ascconf_org;	/* Do not edit value */

static int ReadDelay = 1;
static int WriteDelay = 20;

#define EEP_PBURST   0x0080        /* bit7 */

#define DvcSleepMilliSecond(n)	usleep(n)

/*====================================================================*/
static int
AscWriteEEPCmdReg(int io_base, uchar cmd_reg)
/*
[Name]
	AscWriteEEPCmdReg
[Argument]
	int io_base;	ASC3050/3350 io base address;
	uchar cmd_reg;	ASC3050/3350 command byte;
[Result]
	0:success   -1:write-fail-retry-over
[Description]
	Write comannd to EEPROM Cmd Reg.( +07h )
*/
{
    uchar read_back;
    int retry;

    for (retry = 0; retry < ASC_EEP_MAX_RETRY; retry++) {
	AscSetChipEEPCmd(io_base, cmd_reg);
	DvcSleepMilliSecond(ReadDelay);
	read_back = AscGetChipEEPCmd(io_base);
	DvcSleepMilliSecond(ReadDelay);
	if (read_back == cmd_reg)
	    return 0;
    }
    return -1; // command write error
}
/*====================================================================*/
static int
AscWriteEEPDataReg( int iop_base, ushort data_reg )
/*
[Name]
	AscWriteEEPDataReg
[Argument]
	int io_base;	ASC3050/3350 io base address;
	uchar data_reg;	ASC3050/3350 data byte;
[Result]
	0:success   -1:write-fail-retry-over
[Description]
	Write data to EEPROM Data Reg.( +06h )
*/
{
    ushort read_back;
    int retry;

    for (retry = 0; retry < ASC_EEP_MAX_RETRY; retry++) {
	AscSetChipEEPData(io_base, data_reg);
	DvcSleepMilliSecond(ReadDelay);
	read_back = AscGetChipEEPData(io_base);
	DvcSleepMilliSecond(ReadDelay);
	if (read_back == data_reg)
	    return 0;
    }
    return -3; // data write error
}

/*====================================================================*/
static int
AscReadEEPWord(int io_base, uchar eep_addr, ushort *eep_data)
/*
[Name]
	AscReadEEPWord
[Argument]
	int io_base;		ASC3050/3350 io base address;
	uchar eep_addr;		Address in EEPROM;
	ushort *eep_data;	Read data
[Result]
	0:success   1:write-fail-retry-over
[Description]
	Read word data from EEPROM.
*/
{
	 // EEP Write disable
    if (AscWriteEEPCmdReg(io_base, ASC_EEP_CMD_WRITE_DISABLE) == -1)
	return -1;
	// Set address in EEP
    if (AscWriteEEPCmdReg(io_base, (uchar)(eep_addr | (uchar)ASC_EEP_CMD_READ) ) == -1)
	return -1;
    *eep_data = AscGetChipEEPData(io_base);	// Get data from EEP
    DvcSleepMilliSecond(ReadDelay);

    return 0;
}
/*====================================================================*/
static int
AscWriteEEPWord(int io_base, uchar eep_addr, ushort eep_data)
/*
[Name]
	AscWriteEEPWord
[Argument]
	int io_base;	ASC3050/3350 io base address;
	uchar addr;	Address in EEPROM;
	ushort word_val; Write word
[Result]
	0:success   -1:write-fail-retry-over
[Description]
	Write word data to EEPROM.
        Do not over-write same data.
*/
{
    ushort read_wval;
    int status;

	// read current data & set access address
    if (AscReadEEPWord(io_base, eep_addr, &read_wval) ==-1)
	return -2;	// can not access for EEP
    if (read_wval == eep_data)
	return 0;	// success: cause do not over-write same data

	// EEP Write enable
    if (AscWriteEEPCmdReg(io_base, ASC_EEP_CMD_WRITE_ABLE) == -1)
	return -1;	// can not write
	// set write data
    if ((status = AscWriteEEPDataReg(io_base, eep_data)) == 0) {
	// Set address in EEP
	status = AscWriteEEPCmdReg(io_base, (uchar)(eep_addr | ASC_EEP_CMD_WRITE));
	DvcSleepMilliSecond(WriteDelay);
    }
	// EEP Write disable : must do
    if (AscWriteEEPCmdReg(io_base, ASC_EEP_CMD_WRITE_DISABLE) == -1)
	return -1;

   return status;
}

/*====================================================================*/
static void
SetStdrTb(uchar *pstdr_new, uchar *psync_new, uchar *psync_set,
		 uchar *psync, uchar *pstdr_en)
/*
[Name]
	SetStdrTb
[Argument]
	uchar *pstdr_new;	New value of Sync priod
				priod(0:20M 2:10M 6:5M) + offset
	uchar *psync_new;	New value of Sync enable
				0:disable  1:enable
	uchar *psync_set;	New value of Sync
				0xFF:no change
				0:Async  5:5MB  10:10MB  20:20MB
	From EEP data
	uchar *psync;		priod(0:20M 2:10M 6:5M) + offset
	uchar *pstdr_en;	EEP value of Sync enable
				0:disable  1:enable
[Description]
	Set STDR priod + offset table (psync_new).
	Set Sync/Async (pstdr_new).
*/
{
    int scsi_id;

    for (scsi_id = 0; scsi_id < 8; scsi_id++) {
	switch(psync_set[scsi_id]) {
	case 20: // 20MB/sec
	    pstdr_new[scsi_id] = (uchar)(psync[scsi_id] & 0x0F);
	    psync_new[scsi_id] = 1;
	    break;
	case 10: // 10MB/sec
	    pstdr_new[scsi_id] = (uchar)((psync[scsi_id] & 0x0F) | 0x20);
	    psync_new[scsi_id] = 1;
	    break;
	case 5: // 5MB/sec
	    pstdr_new[scsi_id] = (uchar)((psync[scsi_id] & 0x0F) | 0x60);
	    psync_new[scsi_id] = 1;
	    break;
	case 0: // Async
	    pstdr_new[scsi_id] = psync[scsi_id];
	    psync_new[scsi_id] = 0;
	    break;
	case 0xFF: // no change
	    pstdr_new[scsi_id] = psync[scsi_id];
	    psync_new[scsi_id] = pstdr_en[scsi_id];
	    break;
	}
    }
}
/*====================================================================*/
static void
MakeDiscSyncBit(ushort *pDiscSync,
		uchar *pdisc_new, uchar *pstdr_new,
		uchar *pdisc_en, uchar *pstdr_en)
/*
[Name]
	MakeDiscSyncBit
[Argument]
	ushort *pDiscSync;	Bit15-8: disconnect enable btis
				Bit7-0:  sync enable bits
	uchar *pdisc_new;	New value of disconnect enable
				0xFF:no change
				0:disable  1:enable
	uchar *pstdr_new;	New value of Sync enable
				0:disable  1:enable
	uchar *pdisc_en;	EEP value of disconnect enable
				0:disable  1:enable
	uchar *pstdr_en;	EEP value of Sync/Async
				0: Sync disable
				1: enable
*/
{
    int scsi_id, disc_bit, sync_bit;

    for (scsi_id = 0, disc_bit = 0x0100, sync_bit = 0x0001, *pDiscSync = 0;
		scsi_id < 8;
		scsi_id++, disc_bit = disc_bit << 1, sync_bit = sync_bit << 1) {
	if (pdisc_new[scsi_id] == 0xFF) // no change
	    pdisc_new[scsi_id] = pdisc_en[scsi_id]; // from EEP
	if (pstdr_new[scsi_id] == 0xFF) // no change
	    pstdr_new[scsi_id] = pstdr_en[scsi_id]; // from EEP
        if (dbgf == 2) printf("pdisc_new(%02X) ptdr_new(%02X)",
		pdisc_new[scsi_id], pstdr_new[scsi_id]);

	if (pdisc_new[scsi_id])
	    *pDiscSync = *pDiscSync | disc_bit;
	if (pstdr_new[scsi_id])
	    *pDiscSync = *pDiscSync | sync_bit;
        if (dbgf == 2) printf(" : pDiscSync(%04X)\n", *pDiscSync);
    }
}
/*====================================================================*/
static int
WriteDiscSyncBit(uchar *pmode, uchar *pdisc, ASCCONF_T *ascconf_p)

/*
[Name]
	WriteDiscSyncBit
[Argument]
	uchar *pmode;		20, 10, 5, 0(=Async)
	uchar *pdisc;		0: Disconnect disable
				1: enable
        ASCCONF_T *ascconf_p;   From EEP data
*/

{
    int eep_addr;
    ushort eep_data;
    int status = 0;

    uchar sync_new[8], stdr_new[8];
    int scsi_id;

        // PCI Burst bit
    if (pburst_chg) {
        if (pburst_val)
            eep_data = ascconf_p->eep00 | EEP_PBURST;
        else
            eep_data = ascconf_p->eep00 & ~(EEP_PBURST);
        if ((status = AscWriteEEPWord(io_base, eep_addr = 0x00, eep_data)) != 0) {
            printf("EEP(%02X) write error : %d\n", eep_addr, status);
            return status;
        }
    }

    if (dbgf == 3) {
	for (scsi_id = 0; scsi_id < 8; scsi_id++)
	    printf("Sync(%02X) : Disc(%02X)\n",
				pmode[scsi_id], pdisc[scsi_id]);
    }

    SetStdrTb(sync_new, stdr_new, pmode, ascconf_p->sync_v, ascconf_p->stdr_v);
    MakeDiscSyncBit(&eep_data, pdisc, pmode, ascconf_p->disc_v, ascconf_p->stdr_v);

        // Disconnect enable & Sync enable bit
    if ((status = AscWriteEEPWord(io_base, eep_addr = 0x20, eep_data)) != 0) {
	printf("EEP(%02X) write error : %d\n", eep_addr, status);
	return status;
    }
    if (dbgf == 3) printf("EEP(%02X) : %04X\n", eep_addr, eep_data);
	// Sync priod + offset
    for (scsi_id = 0; scsi_id < 8; scsi_id+= 2) {
	eep_addr = 0x25 + (scsi_id / 2);
	eep_data = (sync_new[scsi_id+1] << 8) | sync_new[scsi_id];
	if (dbgf == 3) printf("EEP(%02X) : %04X\n", eep_addr, eep_data);
	if ((status = AscWriteEEPWord(io_base, eep_addr, eep_data)) != 0) {
	    printf("EEP(%02X) write error : %d\n", eep_addr, status);
	    return status;
	}
    }
    return 0;
}

/*====================================================================*/
static int
GetSyncDiscFormEEP(ASCCONF_T *ascconf_p)
/*
[name]
	GetSyncDiscFormEEP
[argument]
        ASCCONF_T ascconf_p;    From EEP data

[description]

*/

{
    int eep_addr;
    ushort eep_data;
    int status = 0;

    uchar disc_val, stdr_val, chk_bit;
    int scsi_id;

    if ((status = AscReadEEPWord(io_base, eep_addr = 0x00, &eep_data)) !=0) {
        printf("\EEP(%02X) read error : %d\n", eep_addr, status);
	return status;
    }
    ascconf_p->eep00 = eep_data; /* save data */
    ascconf_p->pburst_v = (eep_data & EEP_PBURST) ? 1 : 0; /* 1:on 0: off */

    if ((status = AscReadEEPWord(io_base, eep_addr = 0x20, &eep_data)) !=0) {
	printf("\EEP(%02X) read error : %d\n", eep_addr, status);
	return status;
    }
    disc_val = (uchar)((eep_data & 0xFF00) >> 8);
					// Bit8-15 is disconnect enable bit
    stdr_val = (uchar)(eep_data & 0x00FF); // Bit7-0 is sync enable bit
    for (scsi_id = 0, chk_bit = 0x01; scsi_id < 8;
				scsi_id++, chk_bit = chk_bit << 1) {
	ascconf_p->disc_v[scsi_id] = (disc_val & chk_bit) ? 1 : 0;
	ascconf_p->stdr_v[scsi_id] = (stdr_val & chk_bit) ? 1 : 0;
    }

    for (scsi_id = 0, eep_addr = 0x25; scsi_id < 8;
				scsi_id+=2, eep_addr++) {
	if ((status = AscReadEEPWord(io_base, eep_addr, &eep_data)) !=0) {
	    printf("\EEP(%02X) read error : %d\n", eep_addr, status);
	    return status;
	}
	if (dbgf == 4) printf("EEP(%02X) : %04X\n", eep_addr, eep_data);
	ascconf_p->sync_v[scsi_id] = (uchar)(eep_data & 0x00FF); // priod + offset
        ascconf_p->sync_v[scsi_id+1] = (uchar)((eep_data & 0xFF00) >> 8); // priod + offset
    }
    return status;
}

/*====================================================================*/

static void
DispEEP(ASCCONF_T *ascconf_p)
/*
[name]
	DispEEP
[argument]
        ASCCONF_T ascconf_p;    From EEP data
*/

{
    int scsi_id;

    printf("\nView EEPROM Setting :");
    for (scsi_id = 0; scsi_id < 8; scsi_id++)
        printf(" ID-%01d ", scsi_id);
    printf("\n");

    printf("Enable Disconnect   :");
    for (scsi_id = 0; scsi_id < 8; scsi_id++) {
	if (ascconf_p->disc_v[scsi_id])
	    printf("   on ");
	else
	    printf("  off ");
    }
    printf("\n");

    printf("Synchronous transfer:");
    for (scsi_id = 0; scsi_id < 8; scsi_id++) {
	if (ascconf_p->stdr_v[scsi_id] == 0) {
	    printf(" Async");
	    continue;
	}

	switch (ascconf_p->sync_v[scsi_id] & 0xF0) {
	case 0x60:
	    printf("  5MB ");
	    break;
	case 0x20:
	    printf(" 10MB ");
	    break;
	case 0x00:
	    printf(" 20MB ");
	    break;
	}
    }
    printf("\n");

    printf("PCI Burst = %s\n",
         ((ascconf_p->pburst_v) ? " on" : "off") );

}

/*====================================================================*/
static void
DumpEEP(void)
/*
[name]
	DumpEEP
[description]
	Dump EEPROM data (hex-mode)
*/
{
    int eep_addr;
    ushort eep_data;
    int status = 0;

    printf("Dump EEP ROM\n");
    printf("  ");
    for (eep_addr = 0; eep_addr < 8; eep_addr++)
	printf(" -%02X-", eep_addr);
    printf("\n");

    for (eep_addr = 0; eep_addr < 64; eep_addr++) {
        if (eep_addr % 16 == 0)
	    printf("%02X", eep_addr);
        else if (eep_addr % 8 == 0)
	    printf("  ");

	if ((status = AscReadEEPWord(io_base, eep_addr, &eep_data)) !=0)
	    break;
	printf(":%04X", eep_data);
	if ((eep_addr + 1) % 8 == 0)
		printf("\n");
    }
    if (dbgf) printf("\nstatus = %d\n", status);
}
/*====================================================================*/
static int
GetChipInfo(int io_base)
/*
[name]
	GetChipInfo
[argument]
	int io_base;
*/
{
static struct chip_info {
	ushort id0;
	uchar id1;
	uchar rev;
	ushort status;
	uchar ctrl;
} info;

    info.id0 = inw(io_base); // ID0
    info.id1 = inb(io_base+1); // ID1
    info.rev = inb(io_base+3); // Revision
    info.status = inw(io_base+0x0e); // Chip Status
    info.ctrl = inb(io_base+0x0f); // Chip Control

    if (dbgf) {
	printf("ChipID = %04X-%02X / Revision = %02X\n",
			info.id0, info.id1, info.rev);
	printf("Chip Status = %04X, Chip Control = %02X\n",
			info.status, info.ctrl);
    }
    if (info.id0 == 0xFFFF)
	return -1; // Not configuraion
    return 0;
}

/*====================================================================*/

typedef struct {
    u_short	vendor, device;
    char	*tag;
    char	*name;
} pci_id_t;

pci_id_t pci_id[] = {
    { 0x10cd, 0x1300, "AdvanSys", "AdvanSys ASC3050/ASC3350" },
    { 0x10cd, 0x1200, "AdvanSys", "AdvanSys ASC3050" }
};
#define PCI_COUNT (sizeof(pci_id)/sizeof(pci_id_t))

static void
disp_pci_inf(u_int vendor, u_int device,
			 u_char bus, u_char devfn,
			 u_char irq, u_int io)
{
    static int hdr = 0;

    if (hdr++ == 0)
	printf("\nVendor : Device : Bus : Devfn : PCI slot / func");
    printf("\n %04x  :  %04x  ", vendor, device);
    printf(": %02x  :  %02x   ", bus, devfn);
    printf(":      %02x  /  %02x ", PCI_SLOT(devfn), PCI_FUNC(devfn));
    printf(": %02x  : %08x ", irq, io);
}

static int
pci_probe(int verbose)
/*
[name]
	pci_probe
[argument]
*/
{
    char s[256], *name = NULL;
//    char *t;
    u_int device, vendor, io, i, n;
    u_char bus, devfn, irq;
    FILE *f;

    printf(this_program, this_version);

    if ((f = fopen("/proc/bus/pci/devices", "r")) != NULL) {
	while (fgets(s, 256, f) != NULL) {

	    n = strtoul(s, NULL, 16);
	    bus = (n >> 8); devfn = (n & 0x00ff);
	    n = strtoul(s+5, NULL, 16);
	    vendor = (n >> 16); device = (n & 0xffff);
	    irq = strtoul(s+13, NULL, 16);
	    n = strtoul(s+16, NULL, 16);
	    io = n & 0xfffffff0;
		// pcibios_read_config_dword(bus, devfn, PCI_BASE_ADDRESS_0, &io);
		// pcibios_read_config_byte(bus, devfn, PCI_INTERRUPT_LINE, &irq);

	    if (dbgf)
		disp_pci_inf(vendor, device, bus, devfn, irq, io);

	    for (i = 0; i < PCI_COUNT; i++)
		if ((vendor == pci_id[i].vendor) &&
		    (device == pci_id[i].device)) break;
	    if (i < PCI_COUNT) {
		name = pci_id[i].name;
		break;
	    }
	}
    } else {
	printf("Can't open %s", "/proc/bus/pci/devices");
    }
/* Support OLD Kernel
    } else if ((f = fopen("/proc/pci", "r")) != NULL) {
	while (fgets(s, 256, f) != NULL) {
	    t = strstr(s, "Device id=");
	    if (t) {
		device = strtoul(t+10, NULL, 16);
		t = strstr(s, "Vendor id=");
		vendor = strtoul(t+10, NULL, 16);
		for (i = 0; i < PCI_COUNT; i++)
		    if ((vendor == pci_id[i].vendor) &&
			(device == pci_id[i].device)) break;
	    } else
		for (i = 0; i < PCI_COUNT; i++)
		    if (strstr(s, pci_id[i].tag) != NULL) break;
	    if (i != PCI_COUNT) {
		name = pci_id[i].name;
		break;
	    } else {
		t = strstr(s, "CardBus bridge");
		if (t != NULL) {
		    name = t + 16;
		    t = strchr(s, '(');
		    t[-1] = '\0';
		    break;
		}
	    }
	}
    }
Support OLD Kernel */

    if (name) {
	if (dbgf)
	    disp_pci_inf(vendor, device, bus, devfn, irq, io);
	io_base = io;
	irq_no = irq;
	printf("\n%s found. ", name);
	if (io_base == 0) {
	    printf("Host adapter is not configured.\n");
	    return -ENODEV;
	}
	printf("I/O Base = %04X\n", io_base);
	return 0;
    } else {
	printf("\nNot found host adapter.\n");
	return -ENODEV;
    }
}

static void
usage(char *name)
{
    static char *pusage[] = {
	"\t\t[--disc_all=<on | off>] [--disc[0-7]=<on | off>]\n",
	"\t\t[--sync_all=<sync_value>] [--sync[0-7]=<sync_value>]\n",
        "\t\t[--pburst=<on | off>]\n\n",
	"  -v : display EEP data\n",
	"  -x : display EEP by hex-code data\n",
	"  --disc_all  : Change disconnect enable.",
	" all scsi id 0 thru 7.\n",
	"  --disc[0-7] : Change disconnect enable.",
	" one scsi id\n",
	"\t\ton: enable  off: disable\n",
	"  --sync_all  : Change synchronous transfer parameter.",
	" all scsi id 0 thru 7.\n",
	"  --sync[0-7] : Change disconnect enable.",
	" one scsi id\n",
	"\t\tsync_value : 20 | 10 | 5 | no\n",
        "  --pburst    : PCI Burst. on: enable  off: disable\n",
	"\n",
	NULL
    };

    char **p;

    fprintf(stderr, "usage: %s [-v] [-x]\n", name);
    for (p = pusage; *p != NULL; *p++)
	fprintf(stderr, *p);
}

/*====================================================================*/

static int
SetupOnOffVal(char *on_off, int scsi_id, uchar *disc_val)
/*
[name]
	SetupDiscVal
[argument]
	char *on_off;
	int scsi_id;
	uchar *disc_val;
*/
{
    uchar val;
    int i;

    if (strcasecmp(on_off, "on") == 0)
	val = 1;
    else if (strcasecmp(on_off, "off") == 0)
	val = 0;
    else
	return -1;

    if (scsi_id == -1)
	for (i = 0; i < 8; i++)
	    disc_val[i] = val;
    else
	disc_val[scsi_id] = val;

    return 0;

}
/*====================================================================*/

static int
SetupSyncVal(char *mode, int scsi_id, uchar *sync_val)
/*
[name]
	SetupSyncVal
[argument]
	char *mode;
	int scsi_id;
	uchar *sync_val;
*/
{
    uchar val;
    int i;

    if (strcmp(optarg, "20") == 0)
	val = 20;
    else if (strcmp(optarg, "10") == 0)
 	val = 10;
    else if (strcmp(optarg, "5") == 0)
 	val = 5;
    else if (strcasecmp(optarg, "no") == 0)
 	val = 0;
    else if (strcasecmp(optarg, "async") == 0)
 	val = 0;
    else
	return -1;

    if (scsi_id == -1)
	for (i = 0; i < 8; i++)
	    sync_val[i] = val;
    else
	sync_val[scsi_id] = val;

    return 0;

}

/*====================================================================*/

static struct option long_options[] = {
    { "disc_all", 1, 0, 0 },
    { "disc0", 1, 0, 0 }, { "disc1", 1, 0, 0 },
    { "disc2", 1, 0, 0 }, { "disc3", 1, 0, 0 },
    { "disc4", 1, 0, 0 }, { "disc5", 1, 0, 0 },
    { "disc6", 1, 0, 0 }, { "disc7", 1, 0, 0 },
    { "sync_all", 1, 0, 0 },
    { "sync0", 1, 0, 0 }, { "sync1", 1, 0, 0 },
    { "sync2", 1, 0, 0 }, { "sync3", 1, 0, 0 },
    { "sync4", 1, 0, 0 }, { "sync5", 1, 0, 0 },
    { "sync6", 1, 0, 0 }, { "sync7", 1, 0, 0 },
    { "pburst", 1, 0, 0 },
    { "debug", 2, 0, 0 },
    {  0, 0, 0, 0 }
};

static void
SetOptionLong(int option_index)
{
    extern char *optarg;

    if (dbgf) {
	printf("option %s", long_options[option_index].name);
	if (optarg)
	    printf (" with arg %s", optarg);
	printf("\n");
    }

    switch (option_index) {
    case 0: // disc_all
    case 1: // disc0
    case 2: // disc1
    case 3: // disc2
    case 4: // disc3
    case 5: // disc4
    case 6: // disc5
    case 7: // disc6
    case 8: // disc7
	SetupOnOffVal(optarg, option_index - 1, disc_val);
	disc_chg = 1;	// mark setup discconet
	break;
    case 9: // sync_all
    case 10: // sync0
    case 11: // sync1
    case 12: // sync2
    case 13: // sync3
    case 14: // sync4
    case 15: // sync5
    case 16: // sync6
    case 17: // sync7
	SetupSyncVal(optarg, option_index - 10, sync_val);
	sync_chg = 1;	// mark setup sync mode
	break;
    case 18: // pburst
	SetupOnOffVal(optarg, 0, &pburst_val);
	pburst_chg = 1;	// mark setup PCI Burst
        break;
    case 19: // debug
	dbgf = (optarg) ? atoi(optarg) : 1;
	break;
    }
}

static void
errexit_iopl(void)
{
    if (iopl(0))
	perror("iopl");
    exit(EXIT_FAILURE);
}

/*====================================================================*/

int main(int argc, char *argv[])
{
    int optch, errflg;
    extern char *optarg;
    int option_index = 0;

    int usage_show = 0;
    errflg = 0;

    while ((optch = getopt_long(argc, argv, "t:hvx",
				long_options, &option_index)) != -1) {
	switch (optch) {
	case 0:
	    SetOptionLong(option_index);
	    break;
//	case 't':
//	    printf("optarg = %s\n", optarg);
//	    break;
	case 'h':
	    usage_show = 1; break;
	case 'v':
	    verbose = 1; break;
	case 'x':
	    hex_show = 1; break;
	default:
	    errflg = 1; break;
	}
    }
    if (usage_show || errflg || (optind < argc)) {
	usage(argv[0]);
	exit(EXIT_FAILURE);
    }

	// find host adapter
    if (pci_probe(verbose) != 0)
	exit(EXIT_FAILURE);

	// start io access
    if (iopl(3)) {
	perror("iopl");
	exit(EXIT_FAILURE);
    }

    if (GetChipInfo(io_base) != 0) {
	printf("Driver module does not active.\n");
	printf("You must connect some scsi device, then reboot system.\n\n");
	errexit_iopl();
    }

    if (disc_chg || sync_chg || verbose || pburst_chg) {
	errflg = GetSyncDiscFormEEP(&ascconf_org);
        if (errflg) errexit_iopl();
    }

    if (disc_chg || sync_chg || pburst_chg) {
	if (dbgf) {
	    printf("Current value of EEP");
	    DispEEP(&ascconf_org);
	}
	errflg = WriteDiscSyncBit(sync_val, disc_val, &ascconf_org);
	if (errflg) printf("WriteDiscSyncBit : %d\n", errflg);
	errflg = GetSyncDiscFormEEP(&ascconf_org);
    }
    if (disc_chg || sync_chg || pburst_chg || verbose) {
	DispEEP(&ascconf_org);
    } else if (hex_show) {
	DumpEEP();
    }

	// end io access
    if (iopl(0)) {
	perror("iopl");
	exit(EXIT_FAILURE);
    }

    exit(0);
}
