/*
**
** Filename: eepasc.h
**
*/

#ifndef uint
#define uint unsigned int
#endif
#ifndef ushort
#define ushort unsigned short
#endif
#ifndef uchar
#define uchar unsigned char
#endif
#ifndef ulong
#define ulong unsigned long
#endif

typedef struct asceep_config {

  ushort cfg_lsw;           /* 00 */
  ushort cfg_msw;           /* 01 */
                            /*        for detail see xxxx */
  uchar  init_sdtr;         /* 32L    host initiate SDTR, default 0x00 */
  uchar  disc_enable;       /* 32H    disconnection enabled, default 0xFF */

  uchar  use_cmd_qng;       /* 33L    use command queuing if possible */
                            /*        default 0x00 */
  uchar  start_motor;       /* 33H    send start motor command, default 0x00 */
  uchar  max_total_qng;     /* 34L    maximum total number of queued commands                */
  uchar  max_tag_qng;       /* 34H    maximum number of tag queue command per target id      */
  uchar  bios_scan;         /* 35L    BIOS will try to scan and take over the device         */
                            /*        if BIOS is enabled, one of the bit here should be set  */
                            /*        default 0x01                                           */
                            /*        if zero, BIOS is disabled, but may still               */
                            /*        occupied a address space, depends on bios_addr         */
  uchar  power_up_wait;     /* 35H    BIOS delay number of second when first come up */
  uchar  no_scam;           /* 36L    non-SCAM tolerant device */
  uchar  chip_scsi_id : 4;  /* 36H    the chip SCSI id */
                            /*        default should be 0x80 ( target id 7 ) */
  uchar  isa_dma_speed : 4; /* 36H    high nibble, ISA Chip DMA Speed  */
                            /*        0 - 10MB - default */
                            /*        1 - 7.69 MB */
                            /*        2 - 6.66 MB */
                            /*        3 - 5.55 MB */
                            /*        4 - 5.00 MB */
                            /*        5 - 4.00 MB */
                            /*        6 - 3.33 MB */
                            /*        7 - 2.50 MB */
  uchar  dos_int13_table[ 8 ];
                            /* 37-40  SDTR value to 8 target devices */
                            /*        lower nibble is SDTR offset ( default 0xf )                           */
                            /*        high nibble is SDTR period index ( default 0, 10MB/sec )              */
                            /*        default value will be 0x0F                                            */

  uchar  adapter_info[ 6 ]; /* 41-43  host adapter information  */
  ushort cntl;              /* 44     the control word                                                      */
                            /*        default value is 0xffff                                               */
                            /*                                                                              */
                            /* enabled when bit set, default 0xFF                                           */
                            /*                                                                              */
                            /*  bit 0  set: do not act as initiator                                         */
                            /*              ( acting as target )                                            */
                            /*  bit 1  set: BIOS greater than one giga byte support                         */
                            /*  bit 2  set: BIOS support more than two drives                               */
                            /*              ( DOS 5.0 and later only )                                      */
                            /*  bit 3  set: BIOS do not support removable disk drive                        */
                            /*  bit 4  set:                                                                 */
                            /*  bit 5  set:                                                                 */
                            /*  bit 6  set: do not inquiry logical unit number                              */
                            /*  bit 7  set: interrupt service routine will process more than one queue      */
                            /*  bit 8  set: no local RAM copying verification                               */
                            /*  bit 9  set: reset SCSI bus during initialization                            */
                            /*  bit 10 set: inquiry scsi devices during initialization                      */
                            /*  bit 11 set: no initilization verbose display                                */
                            /*  bit 12 set: SCSI parity enabled                                             */
                            /*  bit 13 set: burst mode disabled                                             */
                            /*  bit 14 set: use only default i/o port address set                           */
                            /*  bit 15 set: */
                            /* for chip version 1 only                                                      */
                            /*        bit 14 set: restore DMA overwritten byte                              */
                            /*        bit 15 set: increment data transfer count by one                      */
                            /*                                                                              */
                            /*    * - always use default setting                                            */
                            /*        do not turn off these bits in normal operation                        */
                            /*        turn off for debuging only !!!                                        */
                            /*                                                                              */
  ushort chksum;            /* 15     check sum of EEPROM                                                   */

}ASCEEP_CONFIG;

#define CSW_HALTED               ( unsigned short )0x0010 /* asc-1000 is currently halted         */

#define ASC_EEP_MAX_ADDR         63 /* maximum word address from zero */

#define INS_HALTINT        ( ushort )0x6281 /* halt with interrupt instruction code */
#define INS_HALT           ( ushort )0x6280 /* halt instruction code */
#define INS_SINT           ( ushort )0x6200 /* set interrupt */
#define INS_RFLAG_WTM      ( ushort )0x7380 /* reset watch dog timer */

#define ASC_EEP_DVC_CFG_BEG      32 /* we use eeprom from index 32 */
#define ASC_EEP_MAX_DVC_ADDR     45 /* number of words used by driver configuration */

#define ASC_EEP_MAX_RETRY        10 /* maximum number of retries write eeprom */
/*
** the EEP command register
*/
#define ASC_EEP_CMD_READ          0x80 /* read operation             */
#define ASC_EEP_CMD_WRITE         0x40 /* write operation            */
#define ASC_EEP_CMD_WRITE_ABLE    0x30 /* enable write opeartion     */
#define ASC_EEP_CMD_WRITE_DISABLE 0x00 /* disable write opeartion    */

#define IOP_CTRL         (0x0F) /* chip control */
#define IOP_STATUS       (0x0E) /* chip status */
#define IOP_INT_ACK      IOP_STATUS /* write only - interrupt ack */

/*
** bank 0, i/o port address
*/
#define IOP_SIG_WORD      (0x00)
#define IOP_SIG_BYTE      (0x01)
#define IOP_CONFIG_LOW    (0x02)
#define IOP_VERSION       (0x03)
#define IOP_CONFIG_HIGH   (0x04)
#define IOP_EEP_DATA      (0x06)
#define IOP_EEP_CMD       (0x07)
#define IOP_RAM_DATA      (0x08)
#define IOP_RAM_ADDR      (0x0A)
#define IOP_SYN_OFFSET    (0x0B)
#define IOP_REG_PC        (0x0C)
#define IOP_EXTRA_CONTROL (0x0D)  /* byte register, begin with PCI ultra chip */
#define IOP_REG_IFC       (0x0D)  /* interface control register, byte */
                                  /* for ASC-1090, ISA PNP, ver number from 33(0x21) */
                                  /* default value = 0x09 */
/*
** bank one, i/o port address
*/
#define IOP_REG_DC1      (0x0E)
#define IOP_REG_DC0      (0x0C)
#define IOP_REG_SB       (0x0B) /* scsi data bus */
#define IOP_REG_DA1      (0x0A)
#define IOP_REG_DA0      (0x08)
#define IOP_REG_SC       (0x09) /* scsi control */
#define IOP_DMA_SPEED    (0x07)
#define IOP_REG_FLAG     (0x07) /* flag */
#define IOP_FIFO_H       (0x06)
#define IOP_FIFO_L       (0x04)
#define IOP_REG_ID       (0x05)
#define IOP_REG_QP       (0x03) /* queue pointer */
#define IOP_REG_IH       (0x02) /* instruction holding register */
#define IOP_REG_IX       (0x01) /* index register */
#define IOP_REG_AX       (0x00) /* accumuloator */

#define ASCV_HALTCODE_W       ( ushort )0x0040 /* halt code */
/*
** chip control
*/
#define CC_CHIP_RESET   ( uchar )0x80 /* reset risc chip  */
#define CC_SCSI_RESET   ( uchar )0x40 /* reset scsi bus   */
#define CC_HALT         ( uchar )0x20 /* halt risc chip   */
#define CC_SINGLE_STEP  ( uchar )0x10 /* single step      */
#define CC_DMA_ABLE     ( uchar )0x08 /* host dma enabled */
#define CC_TEST         ( uchar )0x04 /* test bit         */
#define CC_BANK_ONE     ( uchar )0x02 /* bank switch bit  */
#define CC_DIAG         ( uchar )0x01 /* diagnostic bit   */

/*----- Define Macros -----*/
#define AscGetChipEEPCmd( port )			( uchar )inb( (port)+IOP_EEP_CMD )
#define AscSetChipEEPCmd( port, data )		outb(data, (port)+IOP_EEP_CMD)

#define AscGetChipEEPData( port )			( ushort )inw( (port)+IOP_EEP_DATA )
#define AscSetChipEEPData( port, data )		outw(data, (port)+IOP_EEP_DATA)
#define AscGetChipLramAddr( port )			( ushort )inw( (port)+IOP_RAM_ADDR )
#define AscSetChipLramAddr( port, addr )	outw(addr, (port)+IOP_RAM_ADDR)
#define AscGetChipLramData( port )			( ushort )inw( (port)+IOP_RAM_DATA )
#define AscSetChipLramData( port, data )	outw(data, (port)+IOP_RAM_DATA)

#define iseven_word( val )  ( ( ( ( uint )val) & ( uint )0x0001 ) == 0 )
#define isodd_word( val )   ( ( ( ( uint )val) & ( uint )0x0001 ) != 0 )

#define AscReadChipIH( port )               ( ushort )inw( (port)+IOP_REG_IH )
#define AscWriteChipIH( port, data )        outw(data, (port)+IOP_REG_IH)

#define AscGetChipControl( port )           ( uchar )inb( (port)+IOP_CTRL )
#define AscSetChipControl( port, cc_val )   outb(cc_val, (port)+IOP_CTRL)

#define AscGetChipStatus( port )            ( ushort )inw( (port)+IOP_STATUS )
#define AscSetChipStatus( port, cs_val )    outw(cs_val, (port)+IOP_STATUS)

